/*
  Copyright (c) Members of the EGEE Collaboration. 2009. 
  See http://www.eu-egee.org/partners/ for details on the copyright holders.  

  Licensed under the Apache License, Version 2.0 (the "License"); 
  you may not use this file except in compliance with the License. 
  You may obtain a copy of the License at 
      http://www.apache.org/licenses/LICENSE-2.0 
  Unless required by applicable law or agreed to in writing, software 
  distributed under the License is distributed on an "AS IS" BASIS, 
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
  See the License for the specific language governing permissions and 
  limitations under the License.
 
  WMFactory suite
  
  Authors: Paolo Andreetto <paolo.andreetto@pd.infn.it>
  
  Version info: $Id: WMFactory_cu_suite.cpp,v 1.1.38.1.6.2 2010/04/07 14:02:46 mcecchi Exp $
  Release: $Name: glite-wms-manager_R_3_4_0_6 $
  
 */

#include "WMFactory_cu_suite.h"

using namespace CppUnit;

/* 
    **********************************************************

      Fake WM methods

    ********************************************************** 
*/

void 
WMTest::submit(classad::ClassAd const* request_ad)
{
}

void 
WMTest::cancel(glite::jobid::JobId const& request_id)
{
}

WMImpl* 
create_test_wm()
{
  return new WMTest();
}


/* 
    **********************************************************

      Unit suite methods

    ********************************************************** 
*/

void
WMFactory_test::setUp()
{
  factory = WMFactory::instance();
}

void
WMFactory_test::tearDown()
{
}

void
WMFactory_test::test_create_wm()
{
  string wm_id("test");

  CPPUNIT_ASSERT( factory->register_wm(wm_id,create_test_wm) );

  WMImpl* result = NULL;

  try
  {
    result = factory->create_wm(wm_id);
  }
  catch(NoCreateWMException exception)
  {
    cerr << exception.what() << endl;
  }

  CPPUNIT_ASSERT( result != NULL );
  
  CPPUNIT_ASSERT( factory->unregister_wm(wm_id) );
}

void
WMFactory_test::test_register_wm()
{
  string wm_id("test");

  CPPUNIT_ASSERT( factory->register_wm(wm_id,create_test_wm) );

  factory->unregister_wm(wm_id);

}

void
WMFactory_test::test_unregister_wm()
{
  string wm_id("test");

  if (factory->register_wm(wm_id,create_test_wm)) {

    CPPUNIT_ASSERT( factory->unregister_wm(wm_id) );

  }
}

void
WMFactory_test::test_create_unexisting_wm()
{
  string wm_id("test");

  WMImpl* result = factory->create_wm(wm_id);
}

void
WMFactory_test::test_unregister_unexisting_wm()
{
  string wm_id("test");

  CPPUNIT_ASSERT( !factory->unregister_wm(wm_id) );

}

void
WMFactory_test::test_register_existing_wm()
{
  string wm_id("test");

  if (factory->register_wm(wm_id,create_test_wm)) {

    CPPUNIT_ASSERT( !factory->register_wm(wm_id,create_test_wm) );

  }  
}
